import contextlib
import sys


def check_verbosity(old_fn):
    """
    Checks that the provided verbosity argument is less than or equal to the
    current verbosity level before executing the decorated method.  If this
    check fails, the method is not executed.
    """
    def new_fn(self, *args, **kwargs):
        verbosity = kwargs.pop('verbosity', 0)
        if verbosity <= self._verbosity:
            old_fn(self, *args, **kwargs)
    return new_fn


class StandardStreams(object):
    """
    Class to facilitate writing of log messages to standard streams.
    """
    def __init__(self):
        self._verbosity = 0

    @check_verbosity
    def out(self, msg, newline=True):
        """
        Prints the given message to stdout, including a newline by default.

        Examples:
        >>> out('Test')
        Test
        >>> out('Test', newline=False)
        Test>>>
        """
        sys.stdout.write(msg + ('\n' if newline else ''))

    @check_verbosity
    def err(self, msg, newline=True):
        """
        Prints the given message to stderr, including a newline by default.

        Examples:
        >>> err('Test')
        Test
        >>> err('Test', newline=False)
        Test>>>
        """
        sys.stderr.write(msg + ('\n' if newline else ''))

    @check_verbosity
    def bar(self, msg='', width=70, position=None, stream=None):
        r"""
        Prints a bar text effect to the standard stream specified by `stream`.

        Examples:
        >>> bar('test', width=10)
        == test ==
        >>> bar(width=10)
        ==========
        >>> bar('Richard Dean Anderson is...', position='top', width=50)
        //========= Richard Dean Anderson is... ========\\
        >>> bar('...MacGyver', position='bottom', width=50)
        \\================= ...MacGyver ================//
        """
        width = max(width, len(msg) + 4)
        # Must set this here as opposed to in the argument list so mock tests
        # will work
        stream = stream or sys.stderr

        if msg:
            bar_len = width - len(msg) - 2
        else:
            bar_len = width

        is_odd = bool(bar_len % 2)
        bar_len //= 2

        start_bar = '=' * (bar_len + (1 if is_odd else 0))
        end_bar = '=' * bar_len

        if position == 'top':
            start_bar = '//' + start_bar[2:]
            end_bar = end_bar[:-2] + r'\\'
        elif position == 'bottom':
            start_bar = r'\\' + start_bar[2:]
            end_bar = end_bar[:-2] + '//'

        if msg:
            msg = ' ' + msg + ' '

        stream.write(start_bar + msg + end_bar + '\n')

    def set_verbosity(self, verbosity):
        """
        Sets the current verbosity level.

        Examples:
        >>> set_verbosity(0)
        >>> err('Good Morning', verbosity=0)
        Good Morning
        >>> err('Dave...', verbosity=1)
        >>> set_verbosity(1)
        >>> err('Vietnam!!', verbosity=1)
        Vietnam!!
        """
        self._verbosity = verbosity


standard_streams = StandardStreams()

out = standard_streams.out
err = standard_streams.err
bar = standard_streams.bar
set_verbosity = standard_streams.set_verbosity


class SectionError(Exception):
    pass


@contextlib.contextmanager
def section(msg):
    """
    Context manager that prints a top bar to stderr upon entering and a bottom
    bar upon exiting.  The caption of the top bar is specified by `msg`.  The
    caption of the bottom bar is '...done!' if the context manager exits
    successfully.  If a SectionError is raised inside of the context manager,
    SectionError.message is printed to stderr and the bottom bar caption
    becomes '...skipped.'.
    """
    bar(msg, position='top')
    try:
        yield
    except SectionError as e:
        err(e.message)
        bar('...skipped.', position='bottom')
    else:
        bar('...done!', position='bottom')



from optparse import make_option
from subprocess import CalledProcessError
import os

from django.core.management.base import BaseCommand, CommandError

from backupdb_utils.exceptions import RestoreError
from backupdb_utils.files import get_latest_timestamped_file
from backupdb_utils.settings import BACKUP_DIR, BACKUP_CONFIG
from backupdb_utils.streams import err, set_verbosity, section, SectionError


class Command(BaseCommand):
    help = 'Restores each database in settings.DATABASES from latest db backup.'
    can_import_settings = True

    option_list = BaseCommand.option_list + (
        make_option(
            '--backup-name',
            help=(
                'Name of backup to restore from.  Example: '
                '`--backup-name=mybackup` will restore any backups that look '
                'like "default-mybackup.pgsql.gz".  Defaults to latest '
                'timestamped backup name.'
            ),
        ),
        make_option(
            '--drop-tables',
            action='store_true',
            default=False,
            help=(
                '** EXPERIMENTAL, USE WITH CAUTION ** Drop all tables in '
                'databases before restoring them.  The SQL dumps which are '
                'created by backupdb have drop statements so, usually, this '
                'option is not necessary.'
            ),
        ),
        make_option(
            '--show-output',
            action='store_true',
            default=False,
            help=(
                'Display the output of stderr and stdout (apart from data which '
                'is piped from one process to another) for processes that are '
                'run while restoring databases.  These are command-line '
                'programs such as `psql` or `mysql`.  This can be useful for '
                'understanding how restoration is failing in the case that it '
                'is.'
            ),
        ),
    )

    def handle(self, *args, **options):
        from django.conf import settings

        # Ensure backup dir present
        if not os.path.exists(BACKUP_DIR):
            raise CommandError("Backup dir '{0}' does not exist!".format(BACKUP_DIR))

        backup_name = options['backup_name']
        drop_tables = options['drop_tables']
        show_output = options['show_output']

        set_verbosity(int(options['verbosity']))

        # Loop through databases
        for db_name, db_config in settings.DATABASES.items():
            with section("Restoring '{0}'...".format(db_name)):
                # Get backup config for this engine type
                engine = db_config['ENGINE']
                backup_config = BACKUP_CONFIG.get(engine)
                if not backup_config:
                    raise SectionError("! Restore for '{0}' engine not implemented".format(engine))

                # Get backup file name
                backup_extension = backup_config['backup_extension']
                if backup_name:
                    backup_file = '{dir}/{db_name}-{backup_name}.{ext}.gz'.format(
                        dir=BACKUP_DIR,
                        db_name=db_name,
                        backup_name=backup_name,
                        ext=backup_extension,
                    )
                else:
                    try:
                        backup_file = get_latest_timestamped_file(backup_extension)
                    except RestoreError as e:
                        raise SectionError('! {0}'.format(e))

                # Find restore command and get kwargs
                restore_func = backup_config['restore_func']
                restore_kwargs = {
                    'backup_file': backup_file,
                    'db_config': db_config,
                    'drop_tables': drop_tables,
                    'show_output': show_output,
                }

                # Run restore command
                try:
                    restore_func(**restore_kwargs)
                    err("* Restored '{db_name}' from '{backup_file}'".format(
                        db_name=db_name,
                        backup_file=backup_file))
                except (RestoreError, CalledProcessError) as e:
                    raise SectionError('! {0}'.format(e))



from django import db
from django.core.management import call_command
from django.utils import unittest

from backupdb.tests.models import TestModel


class BackupDBRestoreDBCommandsTestCase(unittest.TestCase):
    multi_db = True

    def setUp(self):
        self.sqlite = TestModel.objects.using('default')
        self.mysql = TestModel.objects.using('mysql')
        self.postgresql = TestModel.objects.using('postgresql')

    def make_test_records(self):
        """
        Creates some test records.  Fixtures don't seem to work.
        """
        self.sqlite.create(name='TestSqlite1')
        self.sqlite.create(name='TestSqlite2')
        self.sqlite.create(name='TestSqlite3')

        self.mysql.create(name='TestMysql1')
        self.mysql.create(name='TestMysql2')

        self.postgresql.create(name='TestPostgresql1')

    def test_can_backup(self):
        self.make_test_records()

        self.assertEqual(self.sqlite.count(), 3)
        self.assertEqual(self.mysql.count(), 2)
        self.assertEqual(self.postgresql.count(), 1)

        call_command('backupdb')

        self.sqlite.all().delete()
        self.mysql.all().delete()
        self.postgresql.all().delete()

        self.assertEqual(self.sqlite.count(), 0)
        self.assertEqual(self.mysql.count(), 0)
        self.assertEqual(self.postgresql.count(), 0)

    def test_can_restore_from_backup(self):
        self.assertEqual(self.sqlite.count(), 0)
        self.assertEqual(self.mysql.count(), 0)
        self.assertEqual(self.postgresql.count(), 0)

        db.close_connection()

        call_command('restoredb')

        self.assertEqual(
            list(self.sqlite.values_list('id', 'name')),
            [
                (1, u'TestSqlite1'),
                (2, u'TestSqlite2'),
                (3, u'TestSqlite3'),
            ],
        )
        self.assertEqual(
            list(self.mysql.values_list('id', 'name')),
            [
                (1, u'TestMysql1'),
                (2, u'TestMysql2'),
            ],
        )
        self.assertEqual(
            list(self.postgresql.values_list('id', 'name')),
            [
                (1, u'TestPostgresql1'),
            ],
        )



DEBUG = True
TEMPLATE_DEBUG = DEBUG
DEBUG_PROPAGATE_EXCEPTIONS = True

ALLOWED_HOSTS = ['*']

ADMINS = tuple()

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'backupdb_database',
        'TEST_NAME': 'test_backupdb_database',
    },
    'mysql': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'backupdb',
        'USER': 'root',
    },
    'postgresql': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'backupdb',
        'USER': 'backupdb_user',
    },
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    }
}

TIME_ZONE = 'America/Denver'
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

USE_I18N = True
USE_L10N = True

MEDIA_ROOT = ''

MEDIA_URL = ''

SECRET_KEY = 'kygee734_14^(r-p&1j1_&)n1&9-efw0_@&*3nu7)zh%iu%fm1'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',

    'backupdb',
    'backupdb.tests',
)

STATIC_URL = '/static/'



#!/usr/bin/env python

from setuptools import setup, find_packages
import subprocess


version = (0, 5, 4, 'final')


def get_version():
    number = '.'.join(map(str, version[:3]))
    stage = version[3]
    if stage == 'final':
        return number
    elif stage == 'alpha':
        process = subprocess.Popen('git rev-parse HEAD'.split(), stdout=subprocess.PIPE)
        stdout, stderr = process.communicate()
        return number + '-' + stdout.decode('utf-8').strip()[:8]


def get_readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='django-backupdb',
    version=get_version(),
    description='Management commands for backing up and restoring databases in Django.',
    long_description=get_readme(),
    author='Fusionbox programmers',
    author_email='programmers@fusionbox.com',
    keywords='django database backup',
    url='https://github.com/fusionbox/django-backupdb',
    packages=find_packages(exclude=('unit_tests', 'unit_tests_scratch')),
    platforms='any',
    license='Fusionbox',
    test_suite='nose.collector',
    setup_requires=[
        'nose>=1.2.1',
    ],
    tests_require=[
        'nose>=1.2.1',
        'mock>=1.0.1',
        'coverage>=3.6',
    ],
    install_requires=[
        'django>=1.3',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
    ],
)
